@@ -1,12 +1,12 @@
 import logging
-from typing import List, Dict, Set, Union, Optional, Tuple
+from typing import List, Dict, Set, Union, Optional, Tuple, Type
 
 import tensorflow as tf
 from keras.engine.keras_tensor import KerasTensor
 from keras.layers.core import TFOpLambda, SlicingOpLambda
 from tensorflow import keras
 from tensorflow.keras.layers import Cropping2D, Cropping3D, UpSampling2D, UpSampling3D, ZeroPadding2D, ZeroPadding3D, \
-    ConvLSTM2D, Flatten, Layer, InputLayer, Input
+    ConvLSTM2D, Flatten, Layer, InputLayer, Input, MultiHeadAttention
 from tensorflow.python.keras.engine.node import Node
 from tensorflow.python.keras.layers.convolutional import Conv
 from tensorflow.python.keras.layers.pooling import GlobalPooling1D, GlobalPooling2D, GlobalPooling3D
@@ -29,7 +29,8 @@ class ModelConverter:
     :type model: tensorflow.keras.Model
     """
 
-    def __init__(self, model: keras.Model, target_data_format: str, inputs_to_transpose: List[str]):
+    def __init__(self, model: keras.Model, target_data_format: str,
+                 inputs_to_transpose: List[str], custom_layers: Dict[str, Type[tf.keras.layers.Layer]]):
         self.target_data_format: str = target_data_format
         self.inputs_to_transpose: List[str] = inputs_to_transpose
         self.model: keras.Model = model
@@ -43,6 +44,8 @@ class ModelConverter:
         # logger
         self._logger = logging.getLogger(__name__)
 
+        self.custom_layers = custom_layers
+
     def convert_model(self) -> keras.Model:
         """
         Converting model to the requested target_data_format
@@ -136,6 +139,13 @@ class ModelConverter:
             elif converted_layer.symbol == 'expand_dims':
                 transformed_axis = self._convert_axis_channel(call_args[1:][0], last_input_axis_index)
                 call_args = (call_args[0], transformed_axis)
+            elif converted_layer.symbol == 'reshape':
+                old_target_shape = call_args[1:][0]
+                if self.target_data_format == "channels_first":
+                    new_target_shape = [old_target_shape[0], old_target_shape[-1], *old_target_shape[1:-1]]
+                else:
+                    new_target_shape = [old_target_shape[0], *old_target_shape[2:], old_target_shape[1]]
+                call_args = (call_args[0], new_target_shape)
 
         return self._default_call_tf_op_lambda_layer(converted_layer, call_args, call_kwargs, node_input_tensors)
 
@@ -176,15 +186,28 @@ class ModelConverter:
 
         layer_name = current_layer.name
         converted_layer = self._layer_cache.get(layer_name)
+        converted_layer_built = True
         if converted_layer is None:
-            converted_layer = convert_layer(current_layer, self.target_data_format, layer_input_shape, transform_signal)
+            # special case of build layers:
+            converted_layer, converted_layer_built = convert_layer(
+                current_layer, self.target_data_format, layer_input_shape, transform_signal, self.custom_layers)
             self._layer_cache[layer_name] = converted_layer
 
-        if isinstance(converted_layer, TFOpLambda):
-            converted_tensor = self._call_tf_op_lambda_layer(converted_layer, node_input_tensors,
-                                                             current_layer, transform_signal)
+        if converted_layer_built:
+            if isinstance(converted_layer, TFOpLambda):
+                converted_tensor = self._call_tf_op_lambda_layer(converted_layer, node_input_tensors,
+                                                                 current_layer, transform_signal)
+            else:
+                converted_tensor = converted_layer(node_input_tensors)
+        elif isinstance(current_layer, MultiHeadAttention):
+            weights = current_layer.get_weights()
+            converted_tensor = converted_layer(*node_input_tensors)
+            converted_layer.set_weights(weights)
         else:
+            weights = current_layer.get_weights()
             converted_tensor = converted_layer(node_input_tensors)
+            converted_layer.set_weights(weights)
+
         self._tensor_cache[tensor_id] = converted_tensor
         self._update_forward_tensor_transform_signal(converted_tensor, transform_signal=transform_signal,
                                                      is_input_tensor=False)
